@@ -39,11 +39,7 @@
 
 #include "compiler.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <assert.h>
+#include "nctype.h"
 
 #include "nasm.h"
 #include "nasmlib.h"
@@ -145,14 +141,13 @@ static void rdf2_init(void)
     segdata = seg_alloc();
     segbss = seg_alloc();
     if (segtext != 0 || segdata != 2 || segbss != 4)
-        nasm_panic(0,
-              "rdf segment numbers not allocated as expected (%d,%d,%d)",
-              segtext, segdata, segbss);
+        nasm_panic("rdf segment numbers not allocated as expected (%d,%d,%d)",
+                   segtext, segdata, segbss);
     bsslength = 0;
     headerlength = 0;
 }
 
-static int32_t rdf2_section_names(char *name, int pass, int *bits)
+static int32_t rdf2_section_names(char *name, int *bits)
 {
     int i;
     bool err;
@@ -160,8 +155,6 @@ static int32_t rdf2_section_names(char *name, int pass, int *bits)
     int code = -1;
     int reserved = 0;
 
-    (void)pass;
-
     /*
      * Default is 32 bits, in the text segment.
      */
@@ -227,7 +220,7 @@ static int32_t rdf2_section_names(char *name, int pass, int *bits)
         code = 3;
     }
     if (nsegments == RDF_MAXSEGS) {
-        nasm_fatal(0, "reached compiled-in maximum segment limit (%d)",
+        nasm_fatal("reached compiled-in maximum segment limit (%d)",
               RDF_MAXSEGS);
         return NO_SEG;
     }
@@ -235,7 +228,7 @@ static int32_t rdf2_section_names(char *name, int pass, int *bits)
     segments[nsegments].segname = nasm_strdup(name);
     i = seg_alloc();
     if (i % 2 != 0)
-        nasm_panic(0, "seg_alloc() returned odd number");
+        nasm_panic("seg_alloc() returned odd number");
     segments[nsegments].segnumber = i >> 1;
     segments[nsegments].segtype = code;
     segments[nsegments].segreserved = reserved;
@@ -497,7 +490,7 @@ static void membufwrite(int segment, const void *data, int bytes)
             break;
     }
     if (i == nsegments)
-        nasm_panic(0, "can't find segment %d", segment);
+        nasm_panic("can't find segment %d", segment);
 
     if (bytes < 0) {
         b = buf;
@@ -520,7 +513,7 @@ static int getsegmentlength(int segment)
             break;
     }
     if (i == nsegments)
-        nasm_panic(0, "can't find segment %d", segment);
+        nasm_panic("can't find segment %d", segment);
 
     return segments[i].seglength;
 }
@@ -533,13 +526,6 @@ static void rdf2_out(int32_t segto, const void *data,
     uint8_t databuf[8], *pd;
     int seg;
 
-    if (segto == NO_SEG) {
-        if (type != OUT_RESERVE)
-            nasm_error(ERR_NONFATAL,
-                  "attempt to assemble code in ABSOLUTE space");
-        return;
-    }
-
     segto >>= 1;                /* convert NASM segment no to RDF number */
 
     for (seg = 0; seg < nsegments; seg++) {
@@ -576,9 +562,6 @@ static void rdf2_out(int32_t segto, const void *data,
             while (size--)
                 membufwrite(segto, databuf, 1);
     } else if (type == OUT_RAWDATA) {
-        if (segment != NO_SEG)
-            nasm_panic(0, "OUT_RAWDATA with other than NO_SEG");
-
         membufwrite(segto, data, size);
     } else if (type == OUT_ADDRESS) {
         int asize = abs((int)size);
@@ -606,7 +589,7 @@ static void rdf2_out(int32_t segto, const void *data,
         membufwrite(segto, databuf, asize);
     } else if (type == OUT_REL2ADR) {
         if (segment == segto)
-            nasm_panic(0, "intra-segment OUT_REL2ADR");
+            nasm_panic("intra-segment OUT_REL2ADR");
 
         rr.reclen = 8;
         rr.offset = getsegmentlength(segto);    /* current offset */
@@ -638,9 +621,9 @@ static void rdf2_out(int32_t segto, const void *data,
         membufwrite(segto, &rr.offset, -2);
     } else if (type == OUT_REL4ADR) {
         if ((segment == segto) && (globalbits != 64))
-            nasm_panic(0, "intra-segment OUT_REL4ADR");
+            nasm_panic("intra-segment OUT_REL4ADR");
         if (segment != NO_SEG && segment % 2) {
-            nasm_panic(0, "erm... 4 byte segment base ref?");
+            nasm_panic("erm... 4 byte segment base ref?");
         }
 
         rr.type = RDFREC_RELOC; /* type signature */
@@ -712,16 +695,11 @@ static void rdf2_cleanup(void)
     fwriteint16_t(0, ofile);
 }
 
-static int32_t rdf2_segbase(int32_t segment)
-{
-    return segment;
-}
-
 /*
  * Handle RDOFF2 specific directives
  */
 static enum directive_result
-rdf2_directive(enum directive directive, char *value, int pass)
+rdf2_directive(enum directive directive, char *value)
 {
     size_t n;
 
@@ -732,7 +710,7 @@ rdf2_directive(enum directive directive, char *value, int pass)
 	    nasm_error(ERR_NONFATAL, "name size exceeds %d bytes", MODLIB_NAME_MAX);
 	    return DIRR_ERROR;
 	}
-        if (pass == 1) {
+        if (pass_first()) {     /* XXX */
             struct DLLRec r;
             r.type = RDFREC_DLL;
             r.reclen = n + 1;
@@ -746,7 +724,7 @@ rdf2_directive(enum directive directive, char *value, int pass)
 	    nasm_error(ERR_NONFATAL, "name size exceeds %d bytes", MODLIB_NAME_MAX);
 	    return DIRR_ERROR;
 	}
-        if (pass == 1) {
+        if (pass_first()) {     /* XXX */
             struct ModRec r;
             r.type = RDFREC_MODNAME;
             r.reclen = n + 1;
@@ -760,30 +738,27 @@ rdf2_directive(enum directive directive, char *value, int pass)
     }
 }
 
-static void rdf2_filename(char *inname, char *outname)
-{
-    standard_extension(inname, outname, ".rdf");
-}
-
 extern macros_t rdf2_stdmac[];
 
 const struct ofmt of_rdf2 = {
     "Relocatable Dynamic Object File Format v2.0",
     "rdf",
+    ".rdf",
     0,
     64,
     null_debug_arr,
     &null_debug_form,
     rdf2_stdmac,
     rdf2_init,
+    null_reset,
     nasm_do_legacy_output,
     rdf2_out,
     rdf2_deflabel,
     rdf2_section_names,
+    NULL,
     null_sectalign,
-    rdf2_segbase,
+    null_segbase,
     rdf2_directive,
-    rdf2_filename,
     rdf2_cleanup,
     NULL                        /* pragma list */
 };
