@@ -1,4 +1,37 @@
 #!/usr/bin/perl
+## --------------------------------------------------------------------------
+##   
+##   Copyright 1996-2016 The NASM Authors - All Rights Reserved
+##   See the file AUTHORS included with the NASM distribution for
+##   the specific copyright holders.
+##
+##   Redistribution and use in source and binary forms, with or without
+##   modification, are permitted provided that the following
+##   conditions are met:
+##
+##   * Redistributions of source code must retain the above copyright
+##     notice, this list of conditions and the following disclaimer.
+##   * Redistributions in binary form must reproduce the above
+##     copyright notice, this list of conditions and the following
+##     disclaimer in the documentation and/or other materials provided
+##     with the distribution.
+##     
+##     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+##     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
+##     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+##     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+##     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+##     CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+##     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+##     NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+##     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+##     HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+##     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+##     OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+##     EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+##
+## --------------------------------------------------------------------------
+
 #
 # version.pl
 #
@@ -23,13 +56,13 @@
 # NASM_VER		-- whole version number as a string
 #
 # version.mac:
-# __NASM_MAJOR__
-# __NASM_MINOR__
-# __NASM_SUBMINOR__
-# __NASM_PATCHLEVEL__
-# __NASM_SNAPSHOT__
-# __NASM_VERSION_ID__
-# __NASM_VER__
+# __?NASM_MAJOR?__
+# __?NASM_MINOR?__
+# __?NASM_SUBMINOR?__
+# __?NASM_PATCHLEVEL?__
+# __?NASM_SNAPSHOT?__
+# __?NASM_VERSION_ID?__
+# __?NASM_VER?__
 #
 
 ($what) = @ARGV;
@@ -37,7 +70,11 @@
 $line = <STDIN>;
 chomp $line;
 
-undef $man, $min, $smin, $plvl, $tail;
+undef $maj;
+undef $min;
+undef $smin;
+undef $plvl;
+undef $tail;
 $is_rc = 0;
 
 if ( $line =~ /^([0-9]+)\.([0-9]+)(.*)$/ ) {
@@ -60,7 +97,7 @@ if ( $line =~ /^([0-9]+)\.([0-9]+)(.*)$/ ) {
     die "$0: Invalid input format\n";
 }
 
-if ($tail =~ /^\-([0-9]+)/) {
+if ($tail =~ /^\-([0-9]+)$/) {
     $snapshot = $1;
 } else {
     undef $snapshot;
@@ -86,8 +123,13 @@ if ($is_rc) {
 
 $nasm_id = ($nmaj << 24)+($nmin << 16)+($nsmin << 8)+$nplvl;
 
-$mangled_ver = sprintf("%d.%02d.%02d", $nmaj, $nmin, $nsmin);
-$mangled_ver .= '.'.$nplvl if ($nplvl != 0);
+$mangled_ver = sprintf("%d.%02d", $nmaj, $nmin);
+if ($nsmin || $nplvl || defined($snapshot)) {
+    $mangled_ver .= sprintf(".%02d", $nsmin);
+    if ($nplvl || defined($snapshot)) {
+	$mangled_ver .= '.'.$nplvl;
+    }
+}
 ($mtail = $tail) =~ tr/-/./;
 $mangled_ver .= $mtail;
 
@@ -105,15 +147,16 @@ if ( $what eq 'h' ) {
     printf "#define NASM_VER            \"%s\"\n", $line;
     print  "#endif /* NASM_VERSION_H */\n";
 } elsif ( $what eq 'mac' ) {
-    printf "%%define __NASM_MAJOR__ %d\n", $nmaj;
-    printf "%%define __NASM_MINOR__ %d\n", $nmin;
-    printf "%%define __NASM_SUBMINOR__ %d\n", $nsmin;
-    printf "%%define __NASM_PATCHLEVEL__ %d\n", $nplvl;
+    print  "STD: version\n";
+    printf "%%define __?NASM_MAJOR?__ %d\n", $nmaj;
+    printf "%%define __?NASM_MINOR?__ %d\n", $nmin;
+    printf "%%define __?NASM_SUBMINOR?__ %d\n", $nsmin;
+    printf "%%define __?NASM_PATCHLEVEL?__ %d\n", $nplvl;
     if (defined($snapshot)) {
-	printf "%%define __NASM_SNAPSHOT__ %d\n", $snapshot;
+	printf "%%define __?NASM_SNAPSHOT?__ %d\n", $snapshot;
     }
-    printf "%%define __NASM_VERSION_ID__ 0%08Xh\n", $nasm_id;
-    printf "%%define __NASM_VER__ \"%s\"\n", $line;
+    printf "%%define __?NASM_VERSION_ID?__ 0%08Xh\n", $nasm_id;
+    printf "%%define __?NASM_VER?__ \"%s\"\n", $line;
 } elsif ( $what eq 'sed' ) {
     printf "s/\@\@NASM_MAJOR\@\@/%d/g\n", $nmaj;
     printf "s/\@\@NASM_MINOR\@\@/%d/g\n", $nmin;
@@ -130,10 +173,19 @@ if ( $what eq 'h' ) {
     printf "NASM_MINOR_VER=%d\n", $nmin;
     printf "NASM_SUBMINOR_VER=%d\n", $nsmin;
     printf "NASM_PATCHLEVEL_VER=%d\n", $nplvl;
+} elsif ( $what eq 'nsis' ) {
+    printf "!define VERSION \"%s\"\n", $line;
+    printf "!define MAJOR_VER %d\n", $nmin;
+    printf "!define MINOR_VER %d\n", $nmin;
+    printf "!define SUBMINOR_VER %d\n", $nsmin;
+    printf "!define PATCHLEVEL_VER %d\n", $nplvl;
 } elsif ( $what eq 'id' ) {
     print $nasm_id, "\n";	 # Print ID in decimal
 } elsif ( $what eq 'xid' ) {
     printf "0x%08x\n", $nasm_id; # Print ID in hexadecimal
+} elsif ( $what eq 'docsrc' ) {
+    printf "\\M{version}{%s}\n", $line;
+    printf "\\M{subtitle}{version %s}\n", $line;
 } else {
     die "$0: Unknown output: $what\n";
 }
